/*********************************************************************************
**
**   $Id: DataTable.cpp,v 1.8 2004/05/27 11:44:07 croftj Exp $
**   Copyright (c) 2004 Joe Croft <joe@croftj.net>
**   
**   This file is part of ah2.
**
**   ah2 is free software; you can redistribute it and/or modify
**   it under the terms of the GNU General Public License as published by
**   the Free Software Foundation; either version 2 of the License, or
**   (at your option) any later version.
**
**   Foobar is distributed in the hope that it will be useful,
**   but WITHOUT ANY WARRANTY; without even the implied warranty of
**   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**   GNU General Public License for more details.
**
**   You should have received a copy of the GNU General Public License
**   along with Foobar; if not, write to the Free Software
**   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**
*********************************************************************************/

#include <qfont.h>
#include <qfontmetrics.h>
#include <qpainter.h>
#include <qregexp.h>
#include <qsqldatabase.h>
#include <qsqldriver.h>
#include <qsqlrecord.h>
#include <stdio.h>
#include "Data/DataTable.h"

DataTable::DataTable(const QString &table, int rows, QWidget *pParent, const char *pName) :
          QDataTable(pParent, pName)
{
   dbTable = table;
   sqlCursor = new QSqlCursor(dbTable);
   sqlCursor->setMode(QSqlCursor::ReadOnly);
   setSqlCursor(sqlCursor);
   insertMode  = false;
   connect(horizontalHeader(), SIGNAL(clicked(int)), this, SLOT(sortBy(int)));
   connect(this, SIGNAL(beforeInsert(QSqlRecord *)), this, SLOT(nextIdent(QSqlRecord *)));
}

DataTable::DataTable(const DataFields *fields, const QString &table, int rows, QWidget *pParent, const char *pName) :
          QDataTable(pParent, pName)
{
   dbTable = table;
   sqlCursor = new QSqlCursor(dbTable);
   sqlCursor->setMode(QSqlCursor::ReadOnly);
   setSqlCursor(sqlCursor);
   setFields(fields);
   insertMode  = false;
   connect(horizontalHeader(), SIGNAL(clicked(int)), this, SLOT(sortBy(int)));
   connect(this, SIGNAL(beforeInsert(QSqlRecord *)), this, SLOT(nextIdent(QSqlRecord *)));
}


void DataTable::setFields(const DataFields *fields) 
{
   //
   // Iterate through all of the field definitions and 
   // place them into the grid.
   //
   for (int col = 0; fields->dataName != NULL; col++) 
   {
      fieldNames[col] = fields->dataName;
      fieldTypes[QString(fields->dataName)] = fields->propName;
      addColumn(fields->dataName, fields->label);
      QFont f = font();
      QFontMetrics fm(f);
      int fwidth = fm.maxWidth();
      printf("\nElement %d |%s|\n", col, (const char*)fields->dataName);
      fflush(stdout);
      if ( fields->minWidth > 0 ) 
      {
         printf("Setting col %d min width %d, (%d pixels)\n", col, fields->minWidth, fwidth * fields->minWidth);
         setColumnWidth(col, fwidth * fields->minWidth);
      }
      fields++;
   }
   selectRow(0);
   if ( currentRecord() != NULL ) 
   {
      printf("DataTable::setFields():Setting current record\n");
      emit currentChanged(currentRecord());
   }
   else 
   {
      printf("DataTable::setFields():current record NULL\n");
   }
}

void DataTable::sortBy(int section)
{
   printf("DataTable::sortBy():Sorting by section: %d\n", section);
   sortBy(fieldNames[section]);
}

void DataTable::sortBy(const QString field)
{
   if ( sort.count() >= 3 ) 
   {
      printf("DataTable::sortBy():Removing item %d from sort\n", sort.count());
      sort.pop_back();
      fflush(stdout);
   }
   printf("DataTable::sortBy():Prepending field |%s|\n", (const char*)field);
   fflush(stdout);
   sort.prepend(field);
   setSort(sort);
   printf("DataTable::sortBy():refreshing table\n");
   fflush(stdout);
   refresh();
   printf("DataTable::sortBy():return\n");
}

void DataTable::nextIdent(QSqlRecord *r)
{
   QString sql = "select nextval('" + dbTable + "_ident_seq')";
   printf("DataTable::nextIdent():sql = '%s'\n", (const char*)sql);
   fflush(stdout);
   QSqlQuery q(sql);
   if ( q.isActive() ) {
      if ( q.next() ) 
      {
         QString idx = q.value(0).toString();
         printf("DataTable::nextIdent():idx = '%s'\n", (const char*)idx);
         fflush(stdout);
         r->setValue("ident", idx);
      }
   }
}

bool DataTable::beginInsert()
{
   return(QDataTable::beginInsert());
}

void DataTable::paintField(QPainter *p, const QSqlField* field, const QRect &cr, bool f)
{
//   printf("DataTable::paintField():Enter\n");
   if ( ! field ) 
      return;

   QString name = field->name();
//   printf("DataTable::paintField():Field name = %s\n", (const char*)name);
   fflush(stdout);
//   printf("DataTable::paintField():Field type = %s\n", (const char*)fieldTypes[name]);
   fflush(stdout);
   if (fieldTypes[name] == "money") 
   {
      QString s = field->value().toString();
//      printf("DataTable::paintField():Field value = %s\n", (const char*)s);
      fflush(stdout);
      QString s1, s2;

      QRegExp re0("^([0-9]+)$");
      re0.search(s);
      s1 = re0.cap(1);
      s.replace(re0, s1 + ".00");
      
      QRegExp re1("(\\.[0-9])$");
      re1.search(s);
      s1 = re1.cap(1);
      s.replace(re1, s1 + "0");
      
      QRegExp re2("([0-9])([0-9]{3}\\.)");
      re2.setMinimal(true);
      re2.search(s);
      s1 = re2.cap(1);
      s2 = re2.cap(2);
//      printf("DataTable::paintField(): s1 = |%s|, s2 = |%s|\n", (const char*)s1, (const char*)s2);
      s.replace(re2, s1 + "," + s2);

      QRegExp re3("([0-9])([0-9]{3},)");
      re3.setMinimal(true);

      re3.search(s);
      s1 = re3.cap(1);
      s2 = re3.cap(2);
      s.replace(re3, s1 + "," + s2);

      re3.search(s);
      s1 = re3.cap(1);
      s2 = re3.cap(2);
      s.replace(re3, s1 + "," + s2);

      re3.search(s);
      s1 = re3.cap(1);
      s2 = re3.cap(2);
      s.replace(re3, s1 + "," + s2);
      s = "$" + s;

//      printf("DataTable::paintField():drawing text %s\n", (const char*)s);
      fflush(stdout);
      p->drawText(2, 2, cr.width()-4, cr.height()-4, fieldAlignment(field), s);
   }
   else
      QDataTable::paintField(p, field, cr, f);
//   printf("DataTable::paintField():Exit\n");
   fflush(stdout);
}

void DataTable::haveKey( QKeyEvent *e) 
{
   printf("DataTable::haveKey(): Entry, passing key event\n");
   keyPressEvent(e);
}

void DataTable::keyPressEvent(QKeyEvent *e)
{
   printf("DataTable::keyPressEvent(): Entry, have key event\n");
   e->ignore();
}

